import 'package:flutter/material.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';

final countServiceProvider = StateNotifierProvider<CountService, int>((ref) => CountService(42)); // <1>

class CountService extends StateNotifier<int> {
  CountService(int firstValue) : super(firstValue);

  void increment() {
    state++;
  }
}

class SmallerStatefulExample extends StatelessWidget {
  const SmallerStatefulExample({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return const ProviderScope(
      child: Counter(),
    );
  }
}

class Counter extends ConsumerStatefulWidget { // <2>
  const Counter({Key? key}) : super(key: key);

  @override
  ConsumerState<Counter> createState() => _CounterState(); // <3>
}

class _CounterState extends ConsumerState<Counter> { // <4>
  late int firstValue;

  @override
  void initState() {
    super.initState();
    firstValue = ref.read(countServiceProvider); // <5>
  }

  @override
  Widget build(BuildContext context) { // <6>
    final currentValue = ref.watch(countServiceProvider); // <7>

    return Column(
      mainAxisSize: MainAxisSize.min,
      children: [
        ElevatedButton( // <8>
          onPressed: () => ref.read(countServiceProvider.notifier).increment(), // <9>
          child: Text("Increment $firstValue -> $currentValue"),
        ),
      ],
    );
  }
}
